######################################################################
#                                                                    #
#                           Moca                                     #
#                                                                    #
#          Pierre Weis, INRIA Rocquencourt                           #
#          Frdric Blanqui, projet Protheo, INRIA Lorraine          #
#                                                                    #
#  Copyright 2005-2008,                                              #
#  Institut National de Recherche en Informatique et en Automatique. #
#  All rights reserved.                                              #
#                                                                    #
#  This file is distributed under the terms of the Q Public License. #
#                                                                    #
######################################################################

#(* $Id: Distrib.mk,v 1.26.2.2 2008/02/13 17:38:45 weis Exp $ *)

# Just for the $(PACKAGE) maintainers, a Makefile to distribute the
# software.
# This Makefile includes the toplevel Makefile of the application that is
# supposed to define the variables:
# PACKAGE (the name of the system to distribute, e.g. Moca)
# SOFTWARE (the name of the application or command to distribute, e.g. mocac)

# Usage:
# - you must set the variables WEBSITEDIR and FTPSITEDIR below,
# - verify that the prerequisites below are fulfilled,
# - then type in
# make -f Distrib.mk distribute

# The Makefile builds a tar ball file with the set of source files and moves
# it the $(FTPSITEDIR) directory.
# The Makefile copies the documentation in the $(WEBSITEDIR) directory.

# Pre-requisites:
# - You need a doc_src directory where the documentation has been written,
#   presumably in HTML, with an index.htm file. This directory is copied
#   onto the WEB site directory of the package, as mentioned in
#   $(WEBSITEDIR).
# - The doc directory is supposed to contain
#   * 2 files named eng.htm and fra.htm that respectively contain
#     the English and French documentation,
#   * 2 files copyright.fra.htm and copyright.eng.htm to rule the copyright
#     notice in french and english.

include Makefile
include config/Makefile.config

# The tag to set on files in the CVS repository when releasing
CVSRELEASETAG = $(PACKAGE)-$(MAINVERSION)_$(SUBVERSION)_$(PATCHLEVEL)

# The file that contains the announce of the new relase in the
# relevant mailing lists.
ANNOUNCEFILE = Announce-$(VERSION)

# Where the software files reside on the CVS repository.
PACKAGEROOT = bazar-ocaml/$(PACKAGE)
# The prefix for the package tar ball file
PACKAGEFULLNAME = $(PACKAGE)-$(VERSION)

WEBSITEDIR=/net/yquem/infosystems/www/bazaar/$(PACKAGE)
FTPSITEDIR=/net/yquem/infosystems/ftp/bazaar/$(PACKAGE)
FTPSITEDIR2=/net/pauillac/infosystems/ftp/cristal/caml-light/bazar-ocaml/$(PACKAGE)

.PHONY: fake distribute package website ftp ftp2 tag retag untag \
        announce version \
	documentation release_test release_examples release

fake:
	@echo
	@echo 'This Makefile is used to distribute the software.'
	@echo 'Read it to understand how to use it!'
	@echo

# Distribution of the software
distribute: package website ftp ftp2 tag announce #version

package: clean-all all release_test release_examples documentation release

clean-all::
	cd doc_src; $(MAKE) $@
	$(RM) doc release

documentation:
	# Make documentation and test before release
	(cd doc_src; $(MAKE) all)
	# Build a clean documentation directory for distribution
	(cd doc_src; $(MAKE) docdir)

release_test:
	# Recompile the tests
	echo; echo;
	echo "Compiling tests";
	echo; echo;
	(cd test; $(MAKE) all)
	echo; echo;
	echo "Tests compiled";
	echo; echo;

release_examples:
	# Create the examples directory; copy the example files.
	echo; echo;
	echo "Populating the examples directory.";
	echo; echo;
	(cd test; $(MAKE) examplesdir)
	echo; echo;
	echo "Examples directory populated.";
	echo; echo;

release:
	# Then clean the old stuff
	$(RM) release
	$(MKDIR) release
	# And check out a brand new version,
	# clean it
	# copy the documentation in it
	cd release; \
	  $(CVS) -d "`cat ../CVS/Root`" co -P -r release05 $(PACKAGEROOT); \
	  $(CP) ../doc $(PACKAGEROOT); \
	  $(CP) ../examples $(PACKAGEROOT); \
	  find . -name CVS | xargs $(RM); \
	  find . -name "*cvsignore*" | xargs $(RM); \
	  find . -name "banner*" | xargs $(RM); \
	  find . -name ".depend*" | xargs $(RM); \
	  find . -name "TODO" | xargs $(RM); \
	  find . -name "README.devel" | xargs $(RM); \
	  find . -name Makefile.distrib | xargs $(RM); \
	  $(RM) $(PACKAGEROOT)/doc_src; \
	  $(RM) $(PACKAGEROOT)/test; \
	  $(RM) $(PACKAGEROOT)/coq

website:
	# Build the Web site
	$(RM) $(WEBSITEDIR).new
	$(MKDIR) $(WEBSITEDIR).new
	$(CP) release/$(PACKAGEROOT)/doc/* $(WEBSITEDIR).new;
	$(CHGRP) caml $(WEBSITEDIR).new
	$(CHMOD) $(WEBSITEDIR).new
	$(MV) $(WEBSITEDIR) $(WEBSITEDIR).old
	$(MV) $(WEBSITEDIR).new $(WEBSITEDIR)
	$(RM) $(WEBSITEDIR).old

ftp:
	# Build the FTP site
	# Copy the legalease to $(FTPSITEDIR)
	$(MKDIR) $(FTPSITEDIR)
	$(CP) release/$(PACKAGEROOT)/README $(FTPSITEDIR)
	$(CP) release/$(PACKAGEROOT)/LICENSE $(FTPSITEDIR)
	$(CP) release/$(PACKAGEROOT)/AUTHORS $(FTPSITEDIR)
	# Give the release its versionning name
	cd release; mv $(PACKAGEROOT) bazar-ocaml/$(PACKAGEFULLNAME); \
	# Build the archive and move it in the proper ftp site
	cd release/bazar-ocaml; \
	  $(TARC) $(PACKAGEFULLNAME).tgz $(PACKAGEFULLNAME); \
	  $(MV) $(PACKAGEFULLNAME).tgz $(FTPSITEDIR)
	$(CP) $(FTPSITEDIR)/$(PACKAGEFULLNAME).tgz $(WEBSITEDIR)/

ftp2:
	# Build the FTP2 site
	# Copy the FTP site to $(FTPSITEDIR2)
	$(MKDIR) $(FTPSITEDIR2)
	$(CP) $(FTPSITEDIR)/* $(FTPSITEDIR2)/

# Tagging the version.
tag:
	$(CVS) rtag -R $(CVSRELEASETAG) bazar-ocaml/$(PACKAGE)

# Tagging the version again with the same tag,
# (in case of bug during the distribution process).
retag:
	$(CVS) rtag -R -F $(CVSRELEASETAG) bazar-ocaml/$(PACKAGE)

# Untagging the version (in case of bug in the software code).
untag:
	$(RM) ./release
	$(CVS) rtag -R -d $(CVSRELEASETAG) bazar-ocaml/$(PACKAGE)

# Sending the announce to the mailing lists.
announce:
	mail -n -s "New release of $(PACKAGE)" \
	  caml-list@inria.fr caml-announce@inria.fr < $(ANNOUNCEFILE)

# Automatic handling of versionning
# Should be done with HTMLC
#version:
#	for i in $(PACKAGEVERSIONFILES); do \
#	echo $$i; \
#	$(MV) $$i $$i~; \
#	sed -e '/ersion/s/$(OLDVERSION)/$(VERSION)/' $$i~ | \
#	sed -e '/year/s/$(OLDYEAR)/$(YEAR)/' > $$i; \
#	done
