(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_mli.ml,v 1.17 2008/02/11 23:05:59 weis Exp $ *)

open Printf
open Genr_base
open Pr_otype
open Otype
open Parsetree
;;

let sharing_type_declaration sharing td =
  let commented_rels = comment_relations_in_type_declaration td in
  if sharing then
    Genr_sharing.add_info_arg_to_type_declaration commented_rels
  else commented_rels
;;

let sharing_type_declarations sharing =
  List.map (fun (s, td) -> s, sharing_type_declaration sharing td)
;;

let make_public_sharing_type_declarations sharing =
  List.map
    (fun (s, td) ->
     s, make_public_type_declaration (sharing_type_declaration sharing td))
;;

let genr_private_type_declarations sharing ob ntds =
  bprintf ob "type %a\n;;\n"
    Pr_otype.type_declarations
    (sharing_type_declarations sharing ntds)
;;

let genr_public_type_declarations sharing ob ntds =
  bprintf ob "type %a\n;;\n"
    Pr_otype.type_declarations
    (make_public_sharing_type_declarations sharing ntds);
  Genr_sharing.print_type_private_structure_items ob ntds;
;;

let genr_construction_function_interface ntd ob (cname, cts, _rels, _loc) =
  match cts with
  | [] ->
    bprintf ob "val %s : %a\n;;\n"
      (Genr_base.construction_function_name cname) ptype ntd
  | _ ->
    bprintf ob "val %s : %a -> %a\n;;\n"
      (construction_function_name cname)
      (list " * " core_type) cts
      ptype ntd
;;

let record_construction_function_name tname = "make_" ^ tname
;;

let abbrev_construction_function_name = record_construction_function_name
;;

let abbrev_projection_function_name tname = "from_" ^ tname
;;

let genr_type_interface ob ((tname, td) as ntd) =
  match td.ptype_kind, td.ptype_private with
  | Ptype_variant cds, Asttypes.Private ->
    List.iter (genr_construction_function_interface ntd ob) cds
  | Ptype_record fds, Asttypes.Private ->
    let ctypes = List.map (fun (_label, _mut_flag, ctype, _rels, _loc) -> ctype) fds in
    bprintf ob "val %s : %a -> %a\n;;\n"
      (record_construction_function_name tname)
      (list " * " core_type) ctypes
      ptype ntd
  | Ptype_abstract _rels, Asttypes.Private ->
    begin match td.ptype_manifest with
    | None -> ()
    | Some ctrep ->
      bprintf ob "val %s : %a -> %a\n;;\n"
        (abbrev_construction_function_name tname)
        core_type ctrep
        ptype ntd;
      bprintf ob "external %s : %a -> %a = \"%%identity\"\n;;\n"
        (abbrev_projection_function_name tname)
        ptype ntd
        core_type ctrep
    end
  | Ptype_variant _, Asttypes.Public
  | Ptype_abstract _, Asttypes.Public
  | Ptype_record _, Asttypes.Public -> ()
;;

let genr_eq_interface ob ((n, _td) as ntd) =
  bprintf ob "val eq_%s : %a -> %a -> bool\n;;\n" n ptype ntd ptype ntd
;;

let genr_mli sharing ob ntds =
  genr_private_type_declarations sharing ob ntds;
  List.iter (genr_type_interface ob) ntds;
  List.iter (genr_eq_interface ob) ntds
;;
