(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_nary.ml,v 1.13 2008/02/11 23:05:59 weis Exp $ *)

(** {3 Generation of construction functions for n-ary generators.} *)

open Parsetree
open Code
open Genr_base
open Relation
;;

let genr_clauses_by_ap _fdef =

  (* Generating the relevant clauses of the construction function for
     the generator given by [fdef], using the relations stated by the
     algebraic property [relation]. *)

  function relation ->

  match relation.prel_desc with

  | Rewrite (pat, expr) -> [Genr_expression.genr_rule pat expr]

  | Structure_item _ -> []
    (* Must not fail for structure_items as relations: this allows the
       user to define a comparison for instance. *)
  | Precedence _ -> []
  | Status _ -> []
  | Commutative _
  | Idempotent _
  | Nilpotent _
  | Associative _
  | Neutral _
  | Inverse _
  | Distributive _
  | Absorbing _
  | Division_by_Absorbent _
  | Involutive ->
      Check.error relation.prel_loc
	(Check.Not_supported "For n-ary generators, algebraic relations are")
;;

let flat_map_end f l accu =
  List.rev (
   List.rev_append accu
     (List.fold_left
       (fun ll l -> List.rev_append (f l) ll)
       []
       l)
  )
;;

let flat_map f l = flat_map_end f l [];;

let clauses_of_generator fdef default_clauses =
  let relations = fdef.fd_relations in
  match relations.prels_desc with
  | Prels_none | Prels_commented _ -> default_clauses
  | Prels_begend rels ->
    flat_map_end (genr_clauses_by_ap fdef) rels default_clauses
;;

let genr_construction_function fdef =
  let g = fdef.fd_generator in
  let arg = Tuple (Code.genr_args "x" fdef.fd_arity) in

  let default_clause = Clause (arg, Genr_base.generator1 g arg) in
  let clauses = clauses_of_generator fdef [default_clause] in

  let body = Match (arg, clauses) in

  [construction_function g, [arg], body]
;;

let genr_function = genr_construction_function;;
