(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

open Symb;;

(* builtin equations in moca *)

type side = Left | Right;;

type axiom =
  | Com of symb
  | Ass of symb
  | Nil of symb * symb
  | Idem of symb
  | Neu of side * symb * symb
  | Abs of side * symb * symb
  | Inv of side * symb * symb * symb
  | Dis of side * symb * symb
  | InvDis of side * symb * symb
  | UDis of symb * symb * symb
  | Invol of symb
  | UNil of symb * symb
  | UIdem of symb
  | InvNeu of symb * symb * symb;;

type theory = axiom list;;

val com : symb -> theory;;
val ass : symb -> theory;;
val nil : symb -> symb -> theory;;
val idem : symb -> theory;;
val lneu : symb -> symb -> theory;;
val rneu : symb -> symb -> theory;;
val neu : symb -> symb -> theory;;
val labs : symb -> symb -> theory;;
val rabs : symb -> symb -> theory;;
val abs : symb -> symb -> theory;;
val linv : symb -> symb -> symb -> theory;;
val rinv : symb -> symb -> symb -> theory;;
val inv : symb -> symb -> symb -> theory;;
val ldis : symb -> symb -> theory;;
val rdis : symb -> symb -> theory;;
val dis : symb -> symb -> theory;;
val linvdis : symb -> symb -> theory;;
val rinvdis : symb -> symb -> theory;;
val invdis : symb -> symb -> theory;;
val udis : symb -> symb -> symb -> theory;;
val invol : symb -> theory;;
val unil : symb -> symb -> theory;;
val uidem : symb -> theory;;
val invneu : symb -> symb -> symb -> theory;;

open Rule;;
open Equation;;

val eqnset_of_theory : theory -> EqnSet.t;;
val rulset_of_theory : theory -> RulSet.t;;
