(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

open Term;;
open Rename;;
open Subs;;
open Subterm;;
open Unif;;
open Rule;;
open Equation;;
open Useful;;

(*let add norm ((l,r) as e) es =
  let l = norm l and r = norm r in
    if l = r then es
    else let e = l, r in
      (Debug.output "cp" "add cp" (new_line Equation.bprint) e;
      EqnSet.add e es);;*)

let bprint_pair b (lr1,lr2) =
  Printf.bprintf b "%a\nand %a\n" Rule.bprint lr1 Rule.bprint lr2;;

(* add to acc the critical pairs between (l1,r1) and (l2,r2) *)

let cp norm (l1,r1) acc (l2,r2) =
  (*Debug.output "cp" "look at cps between " bprint_pair ((l1,r1),(l2,r2));*)

  (* rename (l2,r2) with variables away from (l1,r1) *)
  let rho = renaming_aways [l1; r1] in
  let l2 = rename rho l2 and r2 = rename rho r2 in

  (* non-variable subterms of l1 that unifies with l2 *)
  let f t p acc =
    match t with
      | Var _ -> acc
      | App _ as t ->
	  (try
	     let s = mgu l2 t in
	       add_non_trivial norm
		 (apply s r1, replace (apply s l1) p (apply s r2)) acc
	   with Failure _ -> acc) in
  let acc = Subterm.foldi f l1 acc in

  (* non-variable and strict subterms of l2 that unifies with l1 *)
  let f t p acc =
    match t, p with
      | Var _, _ | _, [] -> acc
      | App _ as t, _ ->
	  (try
	     let s = mgu l1 t in
	       add_non_trivial norm
		 (apply s r2, replace (apply s l2) p (apply s r1)) acc
	   with Failure _ -> acc) in
  Subterm.foldi f l2 acc;;

(* critical pairs of a set of rules *)

let add_cps_with norm r1 =
  let rec aux acc = function
    | [] -> acc
    | r :: rs -> aux (cp norm r1 acc r) rs
  in aux;;

let add_cps norm =
  let rec aux acc = function
    | [] -> acc
    | (r1 :: rs) as rules -> aux (add_cps_with norm r1 acc rules) rs
  in aux;;

let add_cps norm acc rs = add_cps norm acc (RulSet.elements rs);;

let cps norm = add_cps norm EqnSet.empty;;
