(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Fr��d��ric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: norm_ml.ml,v 1.3 2008/01/21 15:11:12 blanqui Exp $ *)

open Rule;;
open Term;;
open Printf;;
open Symb;;
open Code;;
open Check;;

let bprint_cons_type ob n =
  if n > 0 then
    begin
      bprintf ob "of t";
      for i = n-1 downto 0 do bprintf ob " * t" done;
      bprintf ob "\n"
    end
;;

let bprint_cons ob s =
  bprintf ob "| %a" bprint_cons_type
    (arity_info (find_gname (string_of_symbol s)))
;;

let bprint_type ob symbs =
  bprintf ob "type t =\n";
  SymbSet.iter (bprint_cons ob) symbs;
  bprintf ob "\n"
;;

let norm_exp = make_Constant "norm"
;;

let bprint_norm_clause ob r =
  let c = match Eqnrel.clause_of_rule r with
    | Clause (l, r) -> Clause (l, Apply (norm_exp, [r]))
    | Clause_when (l, c, r) -> Clause_when (l, c, Apply (norm_exp, [r]))
  in Code.clause ob c
;;

let bprint_norm ob rs =
  bprintf ob "let rec norm = function";
  RulSet.iter (bprint_norm_clause ob) rs;
  bprintf ob "\n| t -> t\n"
;;

let bprint_main ob =
bprintf ob "
let ic = open_in \"term.in\"
let lb = Lexing.from_channel ic in
let t = norm (Cparser.top_term Clexer.next_token lb) in
close_in ic;
let oc = open_out \"term.out\"
and ob = Buffer.create 1000 in
Term.bprint ob t;
Buffer.output_buffer oc ob;
close_out oc"
;;

let ob = Buffer.create 1000
;;

let write_and_close oc =
  Buffer.output_buffer oc ob;
  Buffer.clear ob;
  close_out oc
;;

let write_norm_file rs =
  let symbs = symbols_of_rules rs
  and oc = open_out "norm.ml" in 
    bprint_type ob symbs;
    bprint_norm ob rs;
    bprint_main ob;
    write_and_close oc
;;

let write_term_file t =
  let oc = open_out "term.in" in
    Term.bprint ob t;
    write_and_close oc
;;

let read_term_file () =
  let ic = open_in "term.out" in
  let lb = Lexing.from_channel ic in
  let t = Cparser.top_term Clexer.next_token lb in
    close_in ic;
    t
;;

let norm rs =
  write_norm_file rs;
  let _ = Sys.command "ocamlc -o norm rlib.cma norm.ml" in
  fun t ->
    write_term_file t;
    let _ = Sys.command "norm" in
      read_term_file
;;
