(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* basic functions on terms *)

open Var;;
open Term;;
open Symb;;

(* fold among subterms *)

val fold : (term -> 'a -> 'a) -> term -> 'a -> 'a;;

(* find a subterm satisfying some predicate. raise Not_found otherwise *)

val find : (term -> bool) -> term -> term;;

(* find Some subterm satisfying some predicate or None *)

val find_opt : (term -> 'a option) -> term -> 'a option;;

(* test whether there exits a subterm satisfying some predicate *)

val exists : (term -> bool) -> term -> bool;;

(* positions in a term *)

type position = int list;;

(* subterm at position p *)

val subterm_pos : term -> position -> term;;
  
(* [replace t p u] replaces the subterm of t at position p by u *)

val replace : term -> position -> term -> term;;

(* functions taking into account the position *)

val foldi : (term -> position -> 'a -> 'a) -> term -> 'a -> 'a;;
val findi : (term -> position -> bool) -> term -> term * position;;
val find_opti : (term -> position -> 'a option) -> term -> ('a * position) option;;
val existsi : (term -> position -> bool) -> term -> bool;;
