(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Laura Lowenthal, projet Protheo, INRIA Lorraine           *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_values.mli,v 1.5 2008/01/10 18:06:58 lowenthl Exp $ *)

(** {Generation of values of a given type definition} *)

val genr_values : bool -> int -> int -> Parsetree.type_declaration -> Parsetree.core_type list -> Code.exp list
(** Generates a list of up to n values of the specified type declaration and arguments, *)
 (** each one with the sepecified maximum constructor depth (i.e number of nested constructors) *)
  (** Usage: genr_values rnd n depth td args where *)
  (** rnd: indicates if it should generate random values *)
  (** n: number of values to generate *)
  (** depth: upper bound for constructor depth *)
  (** td, args: type declaration and arguments *)
   (** Generates as many values as possible, up to a maximum of n. *)

val genr_core_values : bool -> int -> int -> Parsetree.core_type -> Code.exp list
(** Generates a list of n values of the specified core type, each one *)
 (** with a maximum depth (number of nested constructors). *)
  (** Usage: genr_values rnd n depth t where *)
  (** rnd: indicates if it should generate random values *)
  (** n: number of values to generate *)
  (** depth: upper bound for constructor depth *)
  (** t: core *)
   (** Generates as many values as possible, up to a maximum of n. *)
;;
