(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Laura Lowenthal, projet Protheo, INRIA Lorraine           *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: tgenr_base.mli,v 1.7 2008/02/13 10:58:59 lowenthl Exp $ *)

(** {Basic functions for value generation.} *)

val is_known_and_varyadic : Parsetree.generator -> 'a list -> bool
(** Decides is a generator (with the arguments it should be applied to) is *)
 (** [is_known_and_varyadic gen args] returns true when *)
 (** - [gen] is defined (by the user) to have a single list as argument *)
  (** - [args] has more than one element. This is for the purpose of test *)
   (** generation only, where generators must be treated as varyadic only if *)
    (** they are applied to many arguments. *)

val is_known_and_varyadic_name : string -> 'a list -> bool
(** Same as above, indicating the generator name instead of the generator *)

val mk_equality : Code.exp -> Code.exp -> Code.exp
(** Creates an expression representing the application of the equality operator *)

val mk_int : int -> Code.exp
(** Creates a value representing an integer. *)
val mk_char : char -> Code.exp
(** Creates a value representing a character. *)
val mk_string : string -> Code.exp
(** Creates a value representing a string. *)
val mk_float : float -> Code.exp
(** Creates a value representing a float. *)
val mk_int32 : int32 -> Code.exp
(** Creates a value representing an int32. *)
val mk_int64 : int64 -> Code.exp
(** Creates a value representing an int 64. *)
val mk_nativeint : nativeint -> Code.exp
(** Creates a value representing an native integer. *)
val mk_tuple : Code.exp list -> Code.exp
(** Creates a value representing a tuple with the specified expressions. *)
val mk_list : Code.exp list -> Code.exp
(** Creates a value representing a list with the specified expressions. *)
;;

val mk_generator : string -> Location.t -> Parsetree.generator
(** Creates a generator with the given name and location *)

val mk_construction : string -> Code.exp list -> Code.exp
(** Creates an expression in Code.exp representing a generator with the *)
 (** specified name and arguments *)
val mk_non_varyadic_construction : string -> Code.exp list -> Code.exp
(** Creates an expression in Code.exp representing a generator with the *)
 (** specified name and arguments. Treats the generator as non vayradic, even if it has a single list argument. *)
;;

val genr_ints : bool -> int -> Code.exp list
(** Creates the specified number of distinct integer expressions. *)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_chars : bool -> int -> Code.exp list
(** Creates the specified number of distinct char expressions.*)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_strings : bool -> int -> Code.exp list
(** Creates the specified number of distinct string expressions.*)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_floats : bool -> int -> Code.exp list
(** Creates the specified number of distinct float expressions.*)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_int32s : bool -> int -> Code.exp list
(** Creates the specified number of distinct int32 expressions.*)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_int64s : bool -> int -> Code.exp list
(** Creates the specified number of distinct int64 expressions.*)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_nativeints : bool -> int -> Code.exp list
(** Creates the specified number of distinct native integer expressions.*)
 (**  Boolean is true if they must be generated randomly, else they are generated sequentially. *)
val genr_units : int -> Code.exp list
(** Creates at most the specified number of distinct unit expressions, i.e., *)
 (** zero or one*)
;;
